/****************************************************************
*Copyright (c) 2018 Tecnalia.
*Licensed under MIT License (the "License");
*Permission is hereby granted, free of charge, to any person obtaining a copy 
of this software and associated documentation files (the "Software"), to deal 
in the Software without restriction, including without limitation the rights 
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
copies of the Software, and to permit persons to whom the Software is furnished 
to do so, subject to the following conditions:
*The above copyright notice and this permission notice shall be included in all 
copies or substantial portions of the Software.
*
*
*THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, 
INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A 
PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT 
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF 
CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE 
OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

*Contributors (in alphabetical order):
*Alberto Molinuevo 								Tecnalia
*Gorka Benguria                                 Tecnalia
*Iñaki Etxaniz                                  Tecnalia
*Juncal Alonso                                  Tecnalia
*Leire Orue-Echevarria                          Tecnalia
*Maria Jose Lopez                               Tecnalia
*Marisa Escalante                               Tecnalia
*
*Initially developed in the context of DECIDE EU project www.DECIDE-h2020.eu
*****************************************************************************/

package eu.decideh2020.acsmi.monitoring.server.api.impl;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.cxf.jaxrs.client.ClientConfiguration;
import org.apache.cxf.jaxrs.client.JAXRSClientFactory;
import org.apache.cxf.jaxrs.client.WebClient;

import com.fasterxml.jackson.jaxrs.json.JacksonJsonProvider;
import com.maxmind.geoip2.DatabaseReader;

import eu.DECIDEh2020.appManager.models.AppDescription;
import eu.DECIDEh2020.mcsla.core.metrics.EvaluationResult;
import eu.DECIDEh2020.mcsla.core.metrics.MetricsContext;
import eu.DECIDEh2020.mcsla.core.metrics.ServiceObjectiveMetrics;
import eu.decideh2020.ACSmI.discovery.backend.client.api.ServiceResourceApi;
import eu.decideh2020.ACSmI.discovery.backend.client.model.ServiceNFRInfoVO;

public class NfrAvailability {
	
	String CheckAvailability (MonitoredVm vm, AppDescription appdescription) {
		String avalue = null;
		String csidvalue = vm.getCsId();
    	String csipvalue = vm.getdockerHostPublicIP();
    	//System.out.println("*****CSID ********" + csidvalue);
    	MetricsContext context = MetricsContext.create(appdescription);
    	//System.out.println("****Metrics context created ********" + context);
    	Map<String, ServiceObjectiveMetrics> metrics = context.getCsMetrics(csidvalue);
    	ServiceObjectiveMetrics availabilityMetrics= metrics.get("Availability");    				
    // Create metrics untill we have InfluxDB
    	long Monitoreduptimebegin=1;
    	Map<String, List<Long>> cloudserviceResults = new HashMap<>();
		//for (int indice=0;indice<4; indice++) {
		//Declare the hashMap to store the monitored values of the Cloud services
		List <Long> MonitoredValuesCs= new ArrayList <Long>();
		//Read the monitored values from the InfluxDB
		long Monitoreduptimebeginms=Monitoreduptimebegin + 1;
		long Monitoreddowntimebeginms=Monitoreduptimebeginms+1;				
		long Monitoredauptimeagainms=Monitoreddowntimebeginms+1;
		MonitoredValuesCs.add(Monitoreduptimebeginms);
		MonitoredValuesCs.add(Monitoreddowntimebeginms);
		MonitoredValuesCs.add(Monitoredauptimeagainms);
		Monitoreduptimebegin=Monitoredauptimeagainms;
		//Add the monitored values to a Hashmap
		cloudserviceResults.put(vm.getCsId(),MonitoredValuesCs);
		String url = vm.getdockerHostPublicIP();
					//		}
		//Assess the monitored values against the MCSLA and write it in the InfluxDB
		EvaluationResult<Double> evaluationresult = availabilityMetrics.evaluate(cloudserviceResults);
		System.out.println ("result Availability: "+evaluationresult.getMeasurementResultValue());
		if (!evaluationresult.isError() && !evaluationresult.isConditionMet()) {
			avalue= "Error in availability";
			System.out.println ("*****Violation occurred****" + avalue);
			System.out.println("Measured value:" + evaluationresult.getMeasurementResultValue());
			return avalue;
    				}
		else {
			avalue = "Availability well";
			//System.out.println ("*****Availability****" + avalue);
			return avalue;
		}
	}     
}


