class ApplicationController < ActionController::Base
  protect_from_forgery with: :exception
  after_filter :allow_iframe_for_all

  private

  def allow_iframe_for_all
    response.headers.delete 'X-Frame-Options'
  end

  def error_response(msg)
    flash[:alert] = msg
    redirect_to '/'
  end

  def render_bad_request
    render(file: "#{Rails.root}/public/400", status: :bad_request)
  end

  def render_contracting_set
    render(file: "#{Rails.root}/public/contracting_set", status: :ok)
  end

  def check_auth
    unless current_user
      flash[:alert] = 'You need to be authorized to access this action'
      redirect_to '/'
    end
  end

  def current_user_session
    return @current_user_session if defined?(@current_user_session)
    @current_user_session = UserSession.find
  end

  def current_user
    return @current_user if defined?(@current_user)
    @current_user = current_user_session && current_user_session.user
  end

  helper_method :current_user_session, :current_user
end
