class ApplicationDescription
  require 'git'
  require 'fileutils'
  require 'exceptions'

  FILE_NAME = 'DECIDE.json'
  REPOS_DIR = 'contracts'
  private_constant :FILE_NAME, :REPOS_DIR

  attr_reader :session, :contract

  def self.storage_path
    File.join(File.expand_path('..', Dir.pwd), REPOS_DIR)
  end

  def initialize(session)
    @session = session
  end

  def get
    Git.init(session.git_access, repo_path)
  end

  def validate
     raise ACSmIExceptions::NoSchemaFound.new unless read['schema'].try(&:any?)
  end

  def read
    ensure_file_exists
    contents = File.read(file_path)
    begin
      JSON.parse(contents)
    rescue JSON::ParserError
      raise 'File contains invalid JSON'
    end
  end

  def write(vms_data)
    ensure_file_exists
    push_changes(vms_data)
    clean_up
  end

  private

  def ensure_file_exists
    Git.init(session.git_access, repo_path) unless Dir.exists?(repo_path)
    raise 'Failed to find application description file' unless File.exists?(file_path)
  end

  def clean_up
    session.remove_git_access
    FileUtils.remove_dir(repo_path)
  end

  def push_changes(vms_data)
    git = Git.new(repo_path)
    git.persist do
      content = JSON.parse File.read(file_path)
      content.merge!(vms_data)
      content = ApplicationDescriptionSchema.fix_output(content)
      File.open(file_path, 'w') { |f| f << JSON.pretty_generate(content) }
    end
  end

  def file_path
    File.join(repo_path, FILE_NAME)
  end

  def repo_path
    File.join(ApplicationDescription.storage_path, session.id)
  end
end