class PlatformConnector
  require 'rest_client'
  require 'rexml/document'

  def self.register_user(email, first_name, last_name, password)
    # use different route for ACSml user registration?
    response = create_request('user', {email: email, 'first_name' => first_name, 'last_name' => last_name,
                                       password: password, 'password_confirmation' => password, phone: '+48000000000',
                                       'user_role_name' => 'standard', limit: 5})
    response['user']['id']
  end

  def self.register_access(resource_id, user_id)
    create_request('access', {resource_id: resource_id, user_id: user_id, use_default_account: 'true', use_default_region: 'true'})
  end

  def self.create_key_pair(user)
    public_key = File.read(File.join(Rails.root, 'db', 'user_key.pub'))
    create_request('key_pair',
                   { name: "acsmi-user-#{user.platform_id}-key", public_key: public_key },
                   user_platform(user.platform_email, user.platform_password))['key_pair']['id']
  end

  def self.access_exists?(region_id, user_id)
    accesses = Hash.from_xml(platform['/accesses.xml'].get)['accesses'] rescue nil
    return false unless accesses
    accesses.select { |access| access['region-id'] == region_id && access['user-id'] == user_id }.any?
  end

  def self.fields_for_credentials(resource_id)
    Hash.from_xml(platform["/resources/#{resource_id}/fields_for_credentials"].get)['fields']
  end

  def self.resource_name_by_id(resource_id)
    begin
      Hash.from_xml(platform["/resources/#{resource_id}.xml"].get)['resource']['name']
    rescue
      raise "Resource with id #{resource_id} was not found"
    end
  end

  def self.resource(resource_id)
    Hash.from_xml(platform["/resources/#{resource_id}.xml"].get)['resource'] rescue nil
  end

  def self.resources
    Hash.from_xml(platform['/resources.xml'].get)['resources']
  end

  def self.instance_type_data(instance_type_id)
    Hash.from_xml(platform["/instance_types/#{instance_type_id}.xml"].get)['instance_type']
  end

  def self.ask_configuration_assistant(request_params)
    # Call to Configuration Assistant on CBP is to follow here

    puts "!!Sending #{request_params}"
    begin
      platform_response = Hash.from_xml(platform['configuration_assistant.xml']
                                            .post(request_params.to_xml(root: 'conditions'), content_type: :xml))
                                            .try(:[], 'configurations').try(&:first) || {}
      puts "!!PR #{platform_response}"

      platform_response
    rescue => e
      if e.respond_to?(:response) && Hash.from_xml(e.response).try(:[], 'errors')
        errors = Hash.from_xml(e.response.body).try(:[], 'errors')

        puts "!!PRRP #{errors}"

        { errors: errors }
      else
        raise "CA failed: #{e.response}; exc: #{e.message}"
      end
    end
  end

  ### stub mapper's api call, replace after corresponding component is finished
  def self.get_instance_data(resource_id)
    inst_data = resource_children_data(resource_id, 'deployment', %w[region_id software_id])
    inst_data ||= {}
    inst_data['instance_type_id'] = resource_children_data(resource_id, 'instance_type', %w[id])
                                        .try(:[], 'id')
    inst_data.select { |_, v| v.present? }
  end

  def self.resource_children_data(resource_id, model_name, fields)
    collection = Hash.from_xml(platform["/#{model_name}s.xml"].get)["#{model_name}s"]
    collection.select! do |o|
      (o['status'] == 'active' || o['active']) && o['resource_id'] == resource_id
    end
    return nil unless collection.any?
    collection.first.select { |k, _| fields.include?(k) }
  end

  class << self

    private

    def platform
      RestClient::Resource.new(
          Setting.value('platform_url'),
          user: Setting.value('platform_email'),
          password: Setting.value('platform_password'),
          timeout: 60,
          open_timeout: 60,
          verify_ssl: true
      )
    end

    def user_platform(email, password)
      RestClient::Resource.new(
          Setting.value('platform_url'),
          user: email,
          password: password,
          timeout: 60,
          open_timeout: 60,
          verify_ssl: true
      )
    end

    def create_request(model_class, attrs = {}, platform_client = platform)
      begin
        response = platform_client["#{model_class.pluralize}/create.xml"].post(attrs.to_xml(root: model_class), content_type: :xml)
      rescue => e
        raise "Create #{model_class} failed: #{response} #{response ? response.body : 'no-response'}; exc: #{e.message}"
      end

      Hash.from_xml(response)
    end
  end
end