/*******************************************************************************
 * Copyright (c) 2017 Tecnalia.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 	
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * 
 * The code has been contributed over a stub generated with the JHipster
 * Application generator http://www.jhipster.tech/.
 * 
 * Contributors (in alphabetical order):
 * 
 * Gorka Benguria                       Tecnalia
 * Iñaki Etxaniz                        Tecnalia
 * Juncal Alonso                        Tecnalia
 * Leire Orue-Echevarria                Tecnalia
 * Maria Jose lopez                     Tecnalia
 * Marisa Escalante                     Tecnalia
 * 
 * Initially developed in the context of DECIDE EU project www.DECIDE-h2020.eu
 ******************************************************************************/
package eu.decideh2020.acsmi.backend.services.server.domain;

import java.io.Serializable;
import java.time.Instant;
import java.time.ZonedDateTime;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.Lob;
import javax.persistence.Table;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Size;

import org.hibernate.annotations.Cache;
import org.hibernate.annotations.CacheConcurrencyStrategy;

import eu.decideh2020.acsmi.backend.services.server.domain.vo.AttributeVO;

/**
 * A Service.
 */
@Entity
@Table(name = "service")
@Cache(usage = CacheConcurrencyStrategy.NONSTRICT_READ_WRITE)
public class Service extends AbstractAuditingEntity implements Serializable, Comparable<Service> {

    private static final long serialVersionUID = 1L;

    @Id
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    private Long id;

    @NotNull
    @Size(max = 70)
    @Column(name = "servicename", length = 70, nullable = false)
    private String servicename;
    
    @Lob
    @Column(name = "firstcontract")
    private byte[] firstcontract;
    
    @Lob
    @Column(name = "secondcontract")
    private byte[] secondcontract;
    
    @Lob
    @Column(name = "thirdcontract")
    private byte[] thirdcontract;
    
    @NotNull
    @Column(name = "serviceclassid", nullable = false)
    private Long serviceclassid;

    @Column(name = "deleted_date")
    private ZonedDateTime deleteddate;
    
    private transient String serviceclassname;
    private transient Double match = 100D;
    private transient List<String> matchingattributelist;
    private transient List<AttributeVO> attributes;
    private transient List<ServiceIncidence> incidences;
    
    
    public Service() {
    	matchingattributelist = new ArrayList<String>();
    	attributes = new ArrayList<AttributeVO>();
    	incidences = new ArrayList<ServiceIncidence>();
    }
    
    public Service(final Long id, final String servicename) {
    	this.id = id;
    	this.servicename = servicename;
    	matchingattributelist = new ArrayList<String>();
    	attributes = new ArrayList<AttributeVO>();
    	incidences = new ArrayList<ServiceIncidence>();
    }
    
    public Service(final Long id, final String servicename,
    		final Long serviceclassid, final String serviceclassname) {
    	this.id = id;
    	this.servicename = servicename;
    	this.serviceclassid = serviceclassid;
    	this.serviceclassname = serviceclassname;
    	matchingattributelist = new ArrayList<String>();
    	attributes = new ArrayList<AttributeVO>();
    	incidences = new ArrayList<ServiceIncidence>();
    }
    
    public Service(final Long id, final String servicename,
    		final Long serviceclassid, final String serviceclassname,
    		final ZonedDateTime deleteddate) {
    	this.id = id;
    	this.servicename = servicename;
    	this.serviceclassid = serviceclassid;
    	this.serviceclassname = serviceclassname;
    	this.deleteddate = deleteddate;
    	matchingattributelist = new ArrayList<String>();
    	attributes = new ArrayList<AttributeVO>();
    	incidences = new ArrayList<ServiceIncidence>();
    }
    
    public Service(final Long id, final String servicename,
    		final Long serviceclassid, final String serviceclassname,
    		final ZonedDateTime deleteddate, final Long incidenceid,
    		final Long incidencetimestamp, final String incidencedetail,
    		final Instant incidencecreateddate, final Instant incidencelastmodifieddate) {
    	this.id = id;
    	this.servicename = servicename;
    	this.serviceclassid = serviceclassid;
    	this.serviceclassname = serviceclassname;
    	this.deleteddate = deleteddate;
    	matchingattributelist = new ArrayList<String>();
    	attributes = new ArrayList<AttributeVO>();
    	
    	incidences = new ArrayList<ServiceIncidence>();
    	incidences.add(new ServiceIncidence(incidenceid,
    			incidencetimestamp, incidencedetail,
    			id, incidencecreateddate, incidencelastmodifieddate));
    }
    

    public Long getId() {
        return id;
    }

    public void setId(Long id) {
        this.id = id;
    }

    public String getServicename() {
        return servicename;
    }

    public Service servicename(String servicename) {
        this.servicename = servicename;
        return this;
    }

    public void setServicename(String servicename) {
        this.servicename = servicename;
    }
    
    public byte[] getFirstcontract() {
        return firstcontract;
    }

    public Service firstcontract(byte[] firstcontract) {
        this.firstcontract = firstcontract;
        return this;
    }

    public void setFirstcontract(byte[] firstcontract) {
        this.firstcontract = firstcontract;
    }
    
    public byte[] getSecondcontract() {
        return secondcontract;
    }

    public Service secondcontract(byte[] secondcontract) {
        this.secondcontract = secondcontract;
        return this;
    }

    public void setSecondcontract(byte[] secondcontract) {
        this.secondcontract = secondcontract;
    }
    
    public byte[] getThirdcontract() {
        return thirdcontract;
    }

    public Service thirdcontract(byte[] thirdcontract) {
        this.thirdcontract = thirdcontract;
        return this;
    }

    public void setThirdcontract(byte[] thirdcontract) {
        this.thirdcontract = thirdcontract;
    }
    
    public Long getServiceclassid() {
        return serviceclassid;
    }

    public Service serviceclassid(Long serviceclassid) {
        this.serviceclassid = serviceclassid;
        return this;
    }

    public void setServiceclassid(Long serviceclassid) {
        this.serviceclassid = serviceclassid;
    }

    public ZonedDateTime getDeleteddate() {
        return deleteddate;
    }

    public Service deleteddate(ZonedDateTime deleteddate) {
        this.deleteddate = deleteddate;
        return this;
    }

    public void setDeleteddate(ZonedDateTime deleteddate) {
        this.deleteddate = deleteddate;
    }
    
    public String getServiceclassname() {
    	return serviceclassname;
    }
    
    public void setServiceclassname(final String serviceclassname) {
    	this.serviceclassname = serviceclassname;
    }
    
    public Double getMatch() {
    	return match;
    }
    
    public void setMatch(final Double match) {
    	this.match = match;
    }
    
    public List<String> getMatchingattributelist() {
    	return matchingattributelist;
    }
    
    public void setMatchingattributelist(final List<String> matchingattributelist) {
    	this.matchingattributelist = matchingattributelist;
    }
    
    public List<AttributeVO> getAttributes() {
    	return attributes;
    }
    
    public void setAttributes(final List<AttributeVO> attributes) {
    	this.attributes = attributes;
    }
    
    public List<ServiceIncidence> getIncidences() {
    	return incidences;
    }
    
    public void setIncidences(final List<ServiceIncidence> incidences) {
    	this.incidences = incidences;
    }


    @Override
    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }
        if (o == null || getClass() != o.getClass()) {
            return false;
        }
        Service service = (Service) o;
        if (service.getId() == null || getId() == null) {
            return false;
        }
        return Objects.equals(getId(), service.getId());
    }

    @Override
    public int hashCode() {
        return Objects.hashCode(getId());
    }

    @Override
    public String toString() {
    	StringBuffer buffer = new StringBuffer("Service{");
    	buffer.append("id=").append(getId());
    	buffer.append(", servicename=").append(getServicename());
    	buffer.append(", serviceclassid=").append(getServiceclassid());
    	buffer.append(", deleteddate=").append(getDeleteddate());
    	buffer.append(", matchingattributelist=[");
    	if (matchingattributelist != null && !matchingattributelist.isEmpty()) {
    		for (String str : matchingattributelist) {
    			buffer.append(str).append(" , ");
    		}
    	}
    	buffer.append("]");
    	buffer.append(", attributes=[");
    	if (attributes != null && !attributes.isEmpty()) {
    		for (AttributeVO attr : attributes) {
    			buffer.append(attr).append(" , ");
    		}
    	}
    	buffer.append("]");
    	buffer.append(", incidences=[");
    	if (incidences != null && !incidences.isEmpty()) {
    		for (ServiceIncidence incidence : incidences) {
    			buffer.append(incidence).append(" , ");
    		}
    	}
    	buffer.append("]");
    	buffer.append(", match=").append(getMatch()).append("}");
    	return buffer.toString();
    }
    
    @Override
    public int compareTo(Service s) {
    	if (match < s.getMatch()) {
    		return 1;
    	}
    	if (match > s.getMatch()) {
    		return -1;
    	}
    	return 0;
    }
}
