/*******************************************************************************
 * Copyright (c) 2017 Tecnalia.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 	
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * 
 * The code has been contributed over a stub generated with the JHipster
 * Application generator http://www.jhipster.tech/.
 * 
 * Contributors (in alphabetical order):
 * 
 * Gorka Benguria                       Tecnalia
 * Iñaki Etxaniz                        Tecnalia
 * Juncal Alonso                        Tecnalia
 * Leire Orue-Echevarria                Tecnalia
 * Maria Jose lopez                     Tecnalia
 * Marisa Escalante                     Tecnalia
 * 
 * Initially developed in the context of DECIDE EU project www.DECIDE-h2020.eu
 ******************************************************************************/
package eu.decideh2020.acsmi.backend.services.server.domain;

import java.io.Serializable;
import java.time.ZonedDateTime;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.Table;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Size;

import org.hibernate.annotations.Cache;
import org.hibernate.annotations.CacheConcurrencyStrategy;

import eu.decideh2020.acsmi.backend.services.server.domain.vo.AttributeVO;

/**
 * A ServiceClass.
 */
@Entity
@Table(name = "service_class")
@Cache(usage = CacheConcurrencyStrategy.NONSTRICT_READ_WRITE)
public class ServiceClass extends AbstractAuditingEntity implements Serializable {

    private static final long serialVersionUID = 1L;

    @Id
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    private Long id;

    @NotNull
    @Size(max = 20)
    @Column(name = "serviceclassname", length = 20, nullable = false)
    private String serviceclassname;

    @Column(name = "deleted_date")
    private ZonedDateTime deleteddate;
    
    private transient List<AttributeVO> commonattributes;
    private transient List<AttributeVO> functionalattributes;
    private transient List<AttributeVO> nonfunctionalattributes;
    
    
    public ServiceClass() {
    	commonattributes = new ArrayList<AttributeVO>();
    	functionalattributes = new ArrayList<AttributeVO>();
    	nonfunctionalattributes = new ArrayList<AttributeVO>();
    }
    

    public Long getId() {
        return id;
    }

    public void setId(Long id) {
        this.id = id;
    }

    public String getServiceclassname() {
        return serviceclassname;
    }

    public ServiceClass serviceclassname(String serviceclassname) {
        this.serviceclassname = serviceclassname;
        return this;
    }

    public void setServiceclassname(String serviceclassname) {
        this.serviceclassname = serviceclassname;
    }

    public ZonedDateTime getDeleteddate() {
        return deleteddate;
    }

    public ServiceClass deleteddate(ZonedDateTime deleteddate) {
        this.deleteddate = deleteddate;
        return this;
    }

    public void setDeleteddate(ZonedDateTime deleteddate) {
        this.deleteddate = deleteddate;
    }
    
    public List<AttributeVO> getCommonattributes() {
        return commonattributes;
    }
    
    public void setCommonattributes(List<AttributeVO> commonattributes) {
        this.commonattributes = commonattributes;
    }
    
    public List<AttributeVO> getFunctionalattributes() {
        return functionalattributes;
    }
    
    public void setFunctionalattributes(List<AttributeVO> functionalattributes) {
        this.functionalattributes = functionalattributes;
    }
    
    public List<AttributeVO> getNonfunctionalattributes() {
        return nonfunctionalattributes;
    }
    
    public void setNonfunctionalattributes(List<AttributeVO> nonfunctionalattributes) {
        this.nonfunctionalattributes = nonfunctionalattributes;
    }


    @Override
    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }
        if (o == null || getClass() != o.getClass()) {
            return false;
        }
        ServiceClass serviceClass = (ServiceClass) o;
        if (serviceClass.getId() == null || getId() == null) {
            return false;
        }
        return Objects.equals(getId(), serviceClass.getId());
    }

    @Override
    public int hashCode() {
        return Objects.hashCode(getId());
    }

    @Override
    public String toString() {
        StringBuffer buffer = new StringBuffer("ServiceClass{");
    	buffer.append("id=").append(getId());
    	buffer.append(", serviceclassname=").append(getServiceclassname());
    	buffer.append(", deleteddate=").append(getDeleteddate());
    	buffer.append(", commonattributes=[");
    	if (commonattributes != null && !commonattributes.isEmpty()) {
    		for (AttributeVO attr : commonattributes) {
    			buffer.append(attr).append(" , ");
    		}
    	}
    	buffer.append("]");
    	buffer.append(", functionalattributes=[");
    	if (functionalattributes != null && !functionalattributes.isEmpty()) {
    		for (AttributeVO attr : functionalattributes) {
    			buffer.append(attr).append(" , ");
    		}
    	}
    	buffer.append("]");
    	buffer.append(", nonfunctionalattributes=[");
    	if (nonfunctionalattributes != null && !nonfunctionalattributes.isEmpty()) {
    		for (AttributeVO attr : nonfunctionalattributes) {
    			buffer.append(attr).append(" , ");
    		}
    	}
    	buffer.append("]}");
    	return buffer.toString();
    }
}
