/*******************************************************************************
 * Copyright (c) 2017 Tecnalia.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 	
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * 
 * The code has been contributed over a stub generated with the JHipster
 * Application generator http://www.jhipster.tech/.
 * 
 * Contributors (in alphabetical order):
 * 
 * Gorka Benguria                       Tecnalia
 * Iñaki Etxaniz                        Tecnalia
 * Juncal Alonso                        Tecnalia
 * Leire Orue-Echevarria                Tecnalia
 * Maria Jose lopez                     Tecnalia
 * Marisa Escalante                     Tecnalia
 * 
 * Initially developed in the context of DECIDE EU project www.DECIDE-h2020.eu
 ******************************************************************************/
package eu.decideh2020.acsmi.backend.services.server.web.rest;

import java.util.List;
import java.util.Optional;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.codahale.metrics.annotation.Timed;

import eu.decideh2020.acsmi.backend.services.server.domain.ServiceClassAttribute;
import eu.decideh2020.acsmi.backend.services.server.repository.ServiceClassAttributeRepository;
import eu.decideh2020.acsmi.backend.services.server.web.rest.util.HeaderUtil;
import io.github.jhipster.web.util.ResponseUtil;

/**
 * REST controller for managing ServiceClassAttribute.
 */
@RestController
@RequestMapping("/api")
public class ServiceClassAttributeResource {

    private final Logger log = LoggerFactory.getLogger(ServiceClassAttributeResource.class);

    private static final String ENTITY_NAME = "serviceClassTypicalAttribute";
        
    private final ServiceClassAttributeRepository serviceClassAttributeRepository;

    public ServiceClassAttributeResource(ServiceClassAttributeRepository serviceClassAttributeRepository) {
        this.serviceClassAttributeRepository = serviceClassAttributeRepository;
    }
    
    /**
     * GET  /service-class-attributes/functional : get the functional ServiceClassAttributes.
     *
     * @return the ResponseEntity with status 200 (OK) and the list of ServiceClassAttributes in body
     */
    @GetMapping("/service-class-attributes")
    @Timed
    public ResponseEntity<List<ServiceClassAttribute>> getAllServiceClassAttributes(@RequestParam String classid) {
    	log.debug("REST request to get the functional ServiceClassAttributes");
        try {
        	List<ServiceClassAttribute> list = null;
        	if (classid != null && classid.trim().length() > 0) {
        		log.debug("getAllServiceClassAttributes  ::  getting the functional ServiceClassAttributes for classid :: " + classid);
        		list = serviceClassAttributeRepository.findAttributes(new Long(classid));
        	}
        	return ResponseEntity.ok(list);
        } catch (Exception e) {
        	log.error("getCommonServiceClassAttributes  ::  Exception  ::  " + e.getMessage());
        	return ResponseEntity.badRequest()
                    .headers(HeaderUtil.createFailureAlert(
                    		ENTITY_NAME, "serviceClassAttributeGetAllError", "Cannot get the list of Service Class Attributes")).body(null);
        }
    }

    /**
     * GET  /service-class-attributes/:id : get the "id" ServiceClassAttributes.
     *
     * @param id the id of the ServiceClassAttributes to retrieve
     * @return the ResponseEntity with status 200 (OK) and with body the ServiceClassAttributes, or with status 404 (Not Found)
     */
    @GetMapping("/service-class-attributes/{id}")
    @Timed
    public ResponseEntity<ServiceClassAttribute> getServiceClassAttribute(@PathVariable Long id) {
        log.debug("REST request to get ServiceClassAttribute : {}", id);
        ServiceClassAttribute serviceClassAttribute = null;
        try {
        	log.debug("getServiceClassAttribute  ::  getting ServiceClassAttribute...");
        	serviceClassAttribute = serviceClassAttributeRepository.findOne(id);
        	
        	log.debug("getServiceClassAttribute  ::  getting OK");
            return ResponseUtil.wrapOrNotFound(Optional.ofNullable(serviceClassAttribute));
        } catch (Exception e) {
        	log.error("getServiceClassAttribute  ::  Exception");
        	return ResponseEntity.badRequest()
                    .headers(HeaderUtil.createFailureAlert(ENTITY_NAME, "serviceClassAttributeGetError", "Cannot get the Service class attribute"))
                    .body(null);
        }
    }
}
