/*******************************************************************************
 * Copyright (c) 2017 Tecnalia.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 	
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * 
 * The code has been contributed over a stub generated with the JHipster
 * Application generator http://www.jhipster.tech/.
 * 
 * Contributors (in alphabetical order):
 * 
 * Gorka Benguria                       Tecnalia
 * Iñaki Etxaniz                        Tecnalia
 * Juncal Alonso                        Tecnalia
 * Leire Orue-Echevarria                Tecnalia
 * Maria Jose Lopez                     Tecnalia
 * Marisa Escalante                     Tecnalia
 * Alberto Molinuevo					Tecnalia
 * 
 * Initially developed in the context of DECIDE EU project www.DECIDE-h2020.eu
 ******************************************************************************/
(function() {
    'use strict';

    angular
        .module('acsmiApp')
        .controller('ServiceDetailController', ServiceDetailController);

    ServiceDetailController.$inject = ['$scope', '$rootScope', '$state', '$stateParams', 'previousState', 'entity', 'Definition'];

    function ServiceDetailController($scope, $rootScope, $state, $stateParams, previousState, entity, Definition) {
        var vm = this;

        vm.service = entity;
        vm.previousState = previousState.name;
        vm.nextState = vm.previousState;
        
        loadDefinitions();
        
        
        // Carga la lista de definiciones
        function loadDefinitions() {
	        Definition.query({},
	        		function success(result) {
						vm.definitions = result;
						loadChecklists();
					},
					function error(error) {}
	        );
        }
        
        
        // Cargar las checklists
        function loadChecklists() {
        	vm.checklists = new Array();
        	for (var i = 0;i < vm.service.attributes.length;i++) {
        		if (vm.service.attributes[i].ischecklist && vm.service.attributes[i].checklistvalue != null) {
        			var arrayValues = vm.service.attributes[i].checklistvalue.split(",");
        			if (arrayValues != null && arrayValues.length > 0) {
        				vm.checklists[vm.service.attributes[i].typeid] = new Array();
        				for (var j = 0;j < arrayValues.length;j++) {
        					for (var k = 0;k < vm.definitions.length;k++) {
        						if (arrayValues[j] == vm.definitions[k].code) {
        							vm.checklists[vm.service.attributes[i].typeid].push(vm.definitions[k].value);
            						break;
        						}
        					}
            			}
        			}
        		}
        	}
        }
        
        
        // Decodifica el fichero PDF de base de datos
        $scope.decode = function(data) {
        	var base64str = data;
			var binary = atob(base64str.replace(/\s/g, ''));
			var len = binary.length;
			var buffer = new ArrayBuffer(len);
			var view = new Uint8Array(buffer);
			for (var i = 0;i < len;i++) {
			    view[i] = binary.charCodeAt(i);
			}
			return view;
        }
        
        
        // Descarga de documentos
        $scope.download = function(index) {
        	var urlCreator = window.URL || window.webkitURL || window.mozURL || window.msURL;
    		if (urlCreator) {
    			var hiddenElement = document.createElement("a");
    			var blob = null;
    			if (index == 0) {
    				blob = new Blob( [$scope.decode(vm.service.firstcontract)], { type: "application/pdf" });
    	        	hiddenElement.download = vm.service.servicename + "_Overarching_services_contract.pdf";
            	} else if (index == 1) {
            		blob = new Blob( [$scope.decode(vm.service.secondcontract)], { type: "application/pdf" });
            		hiddenElement.download = vm.service.servicename + "_SLA_contract.pdf";
            	} else if (index == 2) {
            		blob = new Blob( [$scope.decode(vm.service.thirdcontract)], { type: "application/pdf" });
            		hiddenElement.download = vm.service.servicename + "_Data_processing_contract.pdf";
            	}
    			var url = urlCreator.createObjectURL(blob);
    			hiddenElement.href = url;
    			hiddenElement.target = "_blank";
    	        hiddenElement.style = "display: none";
    	        document.body.appendChild(hiddenElement);
    	        hiddenElement.click();
    	        
    	        setTimeout(function() {window.URL.revokeObjectURL(url), 100});
    		}
        }
        
        
        var unsubscribe = $rootScope.$on('acsmiApp:serviceUpdate', function(event, result) {
            vm.service = result;
        });
        $scope.$on('$destroy', unsubscribe);
    }
})();
