/*******************************************************************************
 * Copyright (c) 2017 Tecnalia.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 	
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * 
 * The code has been contributed over a stub generated with the JHipster
 * Application generator http://www.jhipster.tech/.
 * 
 * Contributors (in alphabetical order):
 * 
 * Gorka Benguria                       Tecnalia
 * Iñaki Etxaniz                        Tecnalia
 * Juncal Alonso                        Tecnalia
 * Leire Orue-Echevarria                Tecnalia
 * Maria Jose lopez                     Tecnalia
 * Marisa Escalante                     Tecnalia
 * 
 * Initially developed in the context of DECIDE EU project www.DECIDE-h2020.eu
 ******************************************************************************/
(function() {
    'use strict';

    angular
        .module('acsmiApp')
        .controller('ProviderMembersDialogController',ProviderMembersDialogController);

    ProviderMembersDialogController.$inject = ['$scope', 'Principal', '$stateParams', '$uibModalInstance', 'entity', 'User', 'JhiLanguageService'];

    function ProviderMembersDialogController ($scope, Principal, $stateParams, $uibModalInstance, entity, User, JhiLanguageService) {
        var vm = this;

        vm.authorities = [];
        vm.userType = [
          'csp', 'pa'
        ];
        vm.clear = clear;
        vm.languages = null;
        vm.save = save;
        vm.user = entity;

        vm.account = null;
        vm.isAuthenticated = null;

        $scope.$on('authenticationSuccess', function() {
            getAccount();
        });

        getAccount();

        function getAccount() {
            Principal.identity().then(function(account) {
                vm.account = account;
                vm.isAuthenticated = Principal.isAuthenticated;
                // Dependiendo del tipo de usuario ofrece ciertos roles
                setRoles(vm.account.userType);
                // Establece los campos 'company', 'users_group' y 'user_type' dentro del objeto que se va a guardar
                vm.user.company = vm.account.company;
                vm.user.usersGroup = vm.account.usersGroup;
                vm.user.userType = vm.account.userType;
                // Establece los campos necesarios dependiendo de la identificación usada, tarjeta de crédito o referencia de contrato
                if (vm.account.userType == "pa") {
                    if (vm.account.creditCardType != null && vm.account.creditCard != null && vm.account.contractReference == null) {
                      vm.user.creditCardType = vm.account.creditCardType;
                      vm.user.creditCard = vm.account.creditCard;
                    } else if (vm.account.creditCardType == null && vm.account.creditCard == null && vm.account.contractReference != null) {
                      vm.user.contractReference = vm.account.contractReference;
                    }
                }
                // Establece el campo 'cif_or_vat' dentro del objeto que se va a guardar
                vm.user.cifOrVat = vm.account.cifOrVat;
            });
        }

        // Dependiendo del tipo de usuario podrá aplicar ciertos roles
        function setRoles(userType) {
            switch (userType) {
              case "csp":
                vm.authorities = [
                  'ROLE_CSP_USER', 'ROLE_CSP_EXTERNAL'
                ];
                break;
              case "pa":
                vm.authorities = [
                  'ROLE_PA_USER'
                ];
                break;
              default:
                // Do nothing
            }
        }

        JhiLanguageService.getAll().then(function (languages) {
            vm.languages = languages;
        });

        function clear () {
            $uibModalInstance.dismiss('cancel');
        }

        function onSaveSuccess (result) {
            vm.isSaving = false;
            $uibModalInstance.close(result);
        }

        function onSaveError () {
            vm.isSaving = false;
        }

        function save () {
            vm.isSaving = true;
            if (vm.user.id !== null) {
                // TODO: add 'ROLE_USER' in a better way
                vm.user.authorities.push("ROLE_USER");
                User.update(vm.user, onSaveSuccess, onSaveError);
            } else {
                // TODO: add 'ROLE_USER' in a better way
                vm.user.authorities.push("ROLE_USER");
                User.save(vm.user, onSaveSuccess, onSaveError);
            }
        }
    }
})();
