/*******************************************************************************
 * Copyright (c) 2017 Tecnalia.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 	
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * 
 * The code has been contributed over a stub generated with the JHipster
 * Application generator http://www.jhipster.tech/.
 * 
 * Contributors (in alphabetical order):
 * 
 * Alberto Molinuevo					Tecnalia
 * Gorka Benguria                       Tecnalia
 * Iñaki Etxaniz                        Tecnalia
 * Juncal Alonso                        Tecnalia
 * Leire Orue-Echevarria                Tecnalia
 * Maria Jose Lopez                     Tecnalia
 * Marisa Escalante                     Tecnalia
 * Alberto Molinuevo					Tecnalia
 * 
 * Initially developed in the context of DECIDE EU project www.DECIDE-h2020.eu
 ******************************************************************************/
(function() {
    'use strict';

    angular
        .module('acsmiApp')
        .controller('SearchController', SearchController);

    SearchController.$inject = ['$scope', 'Principal', '$translate', '$state', '$stateParams', 'ServiceClass', 'Service', 'Definition'];

    function SearchController ($scope, Principal, $translate, $state, $stateParams, ServiceClass, Service, Definition) {
        var vm = this;
        
        vm.searchInvalidValue = null;
        vm.endorseFRInvalidValue = null;
        vm.endorseFRInvalidValueField = null;
        vm.endorseNFRInvalidValue = null;
        vm.endorseNFRInvalidValueField = null;
        
        vm.service = {};
        vm.selectedclassid = null;
        vm.searchselectedclassid = null;
        vm.selectedclass = null;
        vm.searchselectedclass = null;
        
        vm.account = null;
        
        vm.results = new Array();

        // Obtenemos la información del usuario logueado (en caso de estarlo)
        $scope.$on('authenticationSuccess', function() {
            getAccount();
        });

        getAccount();

        function getAccount() {
            Principal.identity().then(function(account) {
                vm.account = account;
            });
        }
        
        
// **************************************** AREA COMÚN **************************************** //
        
        
        loadServiceClasses();
        loadDefinitions();

        
        // Carga la lista de clases de servicio
        function loadServiceClasses() {
	        ServiceClass.fullquery({},
	        		function success(result) {
	        			vm.serviceClasses = result;
	        		},
	        		function error(error) {}
	        );
        }

        
// **************************************** AREA DE BUSCADOR **************************************** //
        
        // Reinicia el objeto de atributo seleccionado
        $scope.resetSelectedAttribute = function() {
        	vm.selectedattribute = {
	   			typeid: null,
	   			childid: null,
	   			name: null,
	   			value: null,
	   			unit: null
	   		};
        }
        
        
        // Envía el formulario de búsqueda
        $scope.submitSearch = function() {
        	// Construir objeto JSON de filtro
        	var filter = {classid: vm.searchselectedclassid, attributes: []};
        	if (vm.filter.length > 0) {
        		for (var i = 0;i < vm.filter.length;i++) {
        			delete vm.filter[i].arrayunits;
        			filter.attributes.push(vm.filter[i]);
        		}
        	}
        	//console.log(JSON.stringify(filter));

        	// Ejecutar consulta
        	Service.find({filter: filter}, function(result) {
        		if (result != null && result.length > 0) {
        			for (var i = 0;i < result.length;i++) {
        				var matchingstring = "";
        				if (result[i].matchingattributelist != null && result[i].matchingattributelist.length > 0) {
        					for (var j = 0;j < result[i].matchingattributelist.length;j++) {
        						if (j == 0) {
        							matchingstring += "[ ";
        						}
        						matchingstring += result[i].matchingattributelist[j];
        						if (j < result[i].matchingattributelist.length - 1) {
        							matchingstring += ", ";
        						} else {
        							matchingstring += " ]";
        						}
        					}
        					result[i].matchingattributelist[0] = matchingstring;
        				}
        			}
        		}
        		vm.results = result;
        	});
        }
        
        
        // Cambia la clase de servicio seleccionada
        $scope.changeClass = function() {
        	// Construir objeto de clase seleccionada
        	for (var i = 0;i < vm.serviceClasses.length;i++) {
        		if (vm.serviceClasses[i].id == vm.searchselectedclassid) {
        			vm.searchselectedclass = vm.serviceClasses[i];
        			break;
        		}
        	}
        	
        	// Reiniciar variables de búsqueda
        	$scope.resetSelectedAttribute();
        	vm.attributes = new Array();
        	vm.filter = new Array();
        	
        	// Cargar array de atributos
        	for (var i = 0;i < vm.searchselectedclass.commonattributes.length;i++) {
        		if (vm.searchselectedclass.commonattributes[i].parentid == null
        				&& !vm.searchselectedclass.commonattributes[i].ischecklist) {
        			vm.attributes.push(vm.searchselectedclass.commonattributes[i]);
        		}
        	}
        	for (var i = 0;i < vm.searchselectedclass.functionalattributes.length;i++) {
        		if (vm.searchselectedclass.functionalattributes[i].parentid == null
        				&& !vm.searchselectedclass.functionalattributes[i].ischecklist) {
        			vm.attributes.push(vm.searchselectedclass.functionalattributes[i]);
        		}
        	}
        	for (var i = 0;i < vm.searchselectedclass.nonfunctionalattributes.length;i++) {
        		if (vm.searchselectedclass.nonfunctionalattributes[i].parentid == null
        				&& !vm.searchselectedclass.nonfunctionalattributes[i].ischecklist) {
        			vm.attributes.push(vm.searchselectedclass.nonfunctionalattributes[i]);
        		}
        	}
        	for (var i = 0;i < vm.attributes.length;i++) {
   				vm.attributes[i].arrayunits = new Array();
   				if (vm.attributes[i].units != null && vm.attributes[i].units.length > 0) {
   					vm.attributes[i].arrayunits = vm.attributes[i].units.split(",");
   				}
    		}
        	
        	// Ejecutar consulta
        	$scope.submitSearch();
        }
        
        
        // Cambia el tipo de atributo seleccionado
        $scope.changeAttributeType = function() {
        	var selectedtypeid = vm.selectedattribute.typeid.split("#")[0];
        	for (var i = 0;i < vm.attributes.length;i++) {
        		if (vm.attributes[i].typeid == selectedtypeid) {
        			vm.selectedattribute.arrayunits = vm.attributes[i].arrayunits;
        			if (vm.selectedattribute.arrayunits.length == 1) {
        				vm.selectedattribute.unit = vm.selectedattribute.arrayunits[0];
        			}
        			break;
        		}
        	}
        	vm.searchInvalidValue = null;
        }
        
        
        // Valida el formato de un valor de un campo
        $scope.validate = function(text) {
        	text = text.replace(",", ".");
        	return isNaN(text);
        }
        
        
        // Añade un atributo al filtro
        $scope.addAttribute = function() {
        	// Descifrar id y nombre del atributo seleccionado
        	var idName = vm.selectedattribute.typeid.split("#");
        	vm.selectedattribute.typeid = idName[0];
        	vm.selectedattribute.childid = idName[1] == null || idName[1].length == 0 ? null : idName[1];
        	vm.selectedattribute.name = idName[2];
        	
        	if (vm.selectedattribute.unit != null && vm.selectedattribute.unit.length > 0 && $scope.validate(vm.selectedattribute.value)) {
        		vm.searchInvalidValue = vm.selectedattribute.value;
        	}
        	
			if (vm.searchInvalidValue == null) {
				// Comprobar si el atributo existe en el filtro. En caso negativo, añadir
				vm.selectedattribute.value = vm.selectedattribute.value.replace(",", ".");
	        	var found = false;
	   			for (var i = 0;i < vm.filter.length;i++) {
	   				if (vm.filter[i].typeid == vm.selectedattribute.typeid) {
	   					vm.filter[i].unit = vm.selectedattribute.unit;
	   					vm.filter[i].value = vm.selectedattribute.value;
	   					found = true;
	   					break;
	   				}
	   			}
	   			if (!found) {
	   				vm.filter.push(vm.selectedattribute);
	   			}
	
	   			// Ejecutar consulta
	   			$scope.submitSearch();
			}

			$scope.resetSelectedAttribute();
        }
        
        
        // Elimina un atributo del filtro
        $scope.removeAttribute = function(index) {
        	vm.filter.splice(index, 1);

        	// Ejecutar consulta
   			$scope.submitSearch();
        }
        
        
// **************************************** AREA DE ALTA DE SERVICIO **************************************** //
        

        // Carga la lista de definiciones
        function loadDefinitions() {
	        Definition.query({},
	        		function success(result) {
						vm.definitions = result;
					},
					function error(error) {}
	        );
        }
        
        
        // Carga la lista de atributos hijos
        $scope.loadChildrenByType = function(definition, array, callingattrid) {
        	for (var j = 0;j < array.length;j++) {
        		// Solo se cargan los hijos de los atributos que son padre
        		if (array[j].parentid != null) {
	        		// Comprobar la carga actual del array de hijos. Si el array de hijos es nulo/vacío, reinicializar
					if (vm.children[array[j].typeid] == null || vm.children[array[j].typeid].length == 0) {
						vm.children[array[j].typeid] = new Array();
					}
					
	        		if (callingattrid == null) {	        			
	        			// Si la llamada no la hace ningún atributo, insertar definiciones
	        			if (array[j].typeid == definition.typeid
	        					&& definition.code.endsWith(vm.service.attributes[array[j].parentid].value.substring(2))) {
		        			vm.children[array[j].typeid].push( {code: definition.code, value: definition.value, typeid: definition.typeid });
	        			}
	        		} else if (array[j].parentid == callingattrid
	        				&& array[j].typeid == definition.typeid
							&& definition.code.endsWith(vm.service.attributes[array[j].parentid].value.substring(2))) {
						// Si la llamada la realiza un atributo, buscar su atributo hijo, y a su vez buscar las definiciones que le corresponden
	        			// A su vez, si el array de hijos contiene definiciones diferentes a las que se necesitan, reinicializar
						if (vm.children[array[j].typeid].length > 0
								&& vm.children[array[j].typeid][0].code.substring(2) != definition.code.substring(2)) {
							vm.children[array[j].typeid] = new Array();
						}
								
						// Insertar definición en el array de hijos
						vm.children[array[j].typeid].push({code: definition.code, value: definition.value, typeid: definition.typeid });
						vm.service.attributes[array[j].typeid].value = null;
	        		}
				}
        	}
        }
        
        
        // Carga el array de atributos checklist
        $scope.loadChecklistArray = function() {
        	for (var i = 0;i < vm.definitions.length;i++) {
        		if (vm.definitions[i].ischeckoption) {
        			// Inicializar array de checklists
        			if (vm.checklists[vm.definitions[i].typeid] == null) {
        				vm.checklists[vm.definitions[i].typeid] = new Array();
					}
        			
        			// Insertar valores en el array general
        			vm.checklists[vm.definitions[i].typeid].push({code: vm.definitions[i].code,
        				value: vm.definitions[i].value, typeid: vm.definitions[i].typeid });
        			
        			// Asignar valores del atributo
        			if (vm.service.attributes[vm.definitions[i].typeid] != null) {
        				if (vm.service.attributes[vm.definitions[i].typeid].arraychecklist == null) {
        					vm.service.attributes[vm.definitions[i].typeid].arraychecklist = new Array();
        				}
            			if (vm.service.id != null && vm.service.attributes[vm.definitions[i].typeid].checklistvalue != null) {
            				vm.service.attributes[vm.definitions[i].typeid].arraychecklist =
            					convertStringToArray(vm.service.attributes[vm.definitions[i].typeid].checklistvalue, ",");
               			}
        			}
        		}
        	}
        }
       	
        
        // Prepara la carga de la lista de atributos hijos y checklists
   		$scope.loadChildren = function(callingattrid, iscommon, isfr, isnonfr) {
   			// Cargar atributos hijos
   			for (var i = 0;i < vm.definitions.length;i++) {
   				if (!vm.definitions[i].ischeckoption) {
   					// Cargar hijos de atributos comunes
   	   				if (iscommon) {
   	  					$scope.loadChildrenByType(vm.definitions[i], vm.selectedclass.commonattributes, callingattrid);
   	   				}
   	   				
   	   				// Cargar hijos de atributos funcionales
   	   				if (isfr) {
   	   					$scope.loadChildrenByType(vm.definitions[i], vm.selectedclass.functionalattributes, callingattrid);
   	   				}   				
   	   				
   	   				// Cargar hijos de atributos no funcionales
   	   				if (isnonfr) {
   	   					$scope.loadChildrenByType(vm.definitions[i], vm.selectedclass.nonfunctionalattributes, callingattrid);
   	   				}
   				}
   			}
   		}
   		
   		
   		// Configura las unidades de un atributo
   		$scope.configUnits = function(attribute) {
   			attribute.arrayunits = new Array();
   			if (attribute.units != null) {
   				attribute.arrayunits = convertStringToArray(attribute.units, ",");
   			}
   			return attribute;
   		}
   		
   		
   		// Carga un array de la lista de atributos
   		$scope.loadAttributesArray = function(array) {
   			for (var i = 0;i < array.length;i++) {
        		vm.service.attributes[array[i].typeid] = array[i];
	        	vm.service.attributes[array[i].typeid].value = null;
	        	vm.service.attributes[array[i].typeid] =
	        		$scope.configUnits(vm.service.attributes[array[i].typeid]);
        	}
   		}
   		
   		
   		// Carga la lista de atributos
        $scope.loadAttributes = function() {
        	for (var i = 0;i < vm.serviceClasses.length;i++) {
        		if (vm.serviceClasses[i].id == vm.selectedclassid) {
        			vm.selectedclass = vm.serviceClasses[i];
        			break;
        		}
        	}
        	
        	// Inicializar arrays
        	vm.children = new Array(new Array());
        	vm.checklists = new Array(new Array());
        	
        	// Cargar variables
        	vm.service.serviceclassid = vm.selectedclass.id;
        	vm.service.serviceclassname = vm.selectedclass.serviceclassname;
            
        	vm.service.attributes = new Array();
        	
        	// Cargar array de checklists
        	$scope.loadChecklistArray();

        	// Cargar arrays de atributos
        	$scope.loadAttributesArray(vm.selectedclass.commonattributes);
        	$scope.loadAttributesArray(vm.selectedclass.functionalattributes);
        	$scope.loadAttributesArray(vm.selectedclass.nonfunctionalattributes);
        	
        	// Resolver información de CSP
        	if (vm.account.userType == 'csp') {
        		for (var i = 0;i < vm.definitions.length;i++) {
        			if (vm.definitions[i].value == vm.account.company) {
        				vm.service.attributes[3].value = vm.definitions[i].code;
        				break;
        			}
        		}
        	}
        }
        
        
        // Realiza el tratamiento de chequeado de una opción en una checklist
        $scope.checkOption = function(typeid, value) {
        	if (vm.service.attributes[typeid].arraychecklist == null) {
        		vm.service.attributes[typeid].arraychecklist = new Array();
        		vm.service.attributes[typeid].arraychecklist.push(value);
        	} else {
        		var index = vm.service.attributes[typeid].arraychecklist.indexOf(value);
        		if (index == -1) {
                	vm.service.attributes[typeid].arraychecklist.push(value);
                } else {
                	vm.service.attributes[typeid].arraychecklist.splice(index, 1);
                }
        	}
        	
        	// Asignar un valor a la propiedad "value" para validar el formulario
       		vm.service.attributes[typeid].value = value;
        }
        
        
        // Valida el formato de un valor de un campo
        $scope.validateEndorseInput = function(units, typeid, isFR) {
        	if (units != null && units.length > 0 && vm.service.attributes[typeid].value != null && vm.service.attributes[typeid].value.length > 0) {
        		vm.service.attributes[typeid].value = vm.service.attributes[typeid].value.replace(",", ".");
        		if (isFR) {
        			vm.endorseFRInvalidValue = null;
        			vm.endorseFRInvalidValueField = null;
            		if (isNaN(vm.service.attributes[typeid].value)) {
            			vm.endorseFRInvalidValueField = vm.service.attributes[typeid].name;
            			vm.endorseFRInvalidValue = vm.service.attributes[typeid].value;
            			vm.service.attributes[typeid].value = null;
            		}
        		} else {
        			vm.endorseNFRInvalidValue = null;
        			vm.endorseNFRInvalidValueField = null;
            		if (isNaN(vm.service.attributes[typeid].value)) {
            			vm.endorseNFRInvalidValueField = vm.service.attributes[typeid].name;
            			vm.endorseNFRInvalidValue = vm.service.attributes[typeid].value;
            			vm.service.attributes[typeid].value = null;
            		}
        		}
        	} else {
        		vm.endorseFRInvalidValue = null;
    			vm.endorseFRInvalidValueField = null;
    			vm.endorseNFRInvalidValue = null;
    			vm.endorseNFRInvalidValueField = null;
        	}
        }
        
        
        // Subida de documentos 
        $scope.addFile = function(elem, index) {
        	var allowedExtensions = ['PDF'];
        	
        	if (elem.files.length > 0) {
        		// Comprobar extensiones permitidas
        		var extensionOk = false;
	        	var extension = elem.files[0].name.split(".").pop().toUpperCase();
	        	for (var i = 0;i < allowedExtensions.length;i++) {
	        		if (extension == allowedExtensions[i]) {
	        			extensionOk = true;
	        			break;
	        		}
	        	}

        		// Validaciones
        		if (!extensionOk) {
        			vm.uploadError = true;
        			angular.element(index).val(null);
        			if (index == 0) {
        				vm.service.firstcontract = null;
        			} else if (index == 1) {
        				vm.service.secondcontract = null;
        			} else if (index == 2) {
        				vm.service.thirdcontract = null;
        			}
        			$scope.$apply();
        		} else {
        			// Si pasa todas las validaciones, subir fichero
        			vm.uploadError = false;
        			var reader = new FileReader();
            		reader.onload = function(e) {            			
            			// Convertir a array de bytes
            			if (index == 0) {
            				vm.service.firstcontract = btoa(e.target.result.toString());
            			} else if (index == 1) {
            				vm.service.secondcontract = btoa(e.target.result.toString());
            			} else if (index == 2) {
            				vm.service.thirdcontract = btoa(e.target.result.toString());
            			}
            			$scope.$apply();
            		};
            		reader.readAsBinaryString(elem.files[0]);
        		}
        	} else {
        		vm.uploadError = false;
        		$scope.removeFile(index);
        	}
        };
        
        
        // Borrado de documentos
        $scope.removeFile = function(index) {
        	if (index == 0) {
        		vm.service.firstcontract = null;
        	} else if (index == 1) {
        		vm.service.secondcontract = null;
        	} else if (index == 2) {
        		vm.service.thirdcontract = null;
        	}
        }
        
        
        // Descarga de documentos
        $scope.download = function(index) {
        	var a = document.createElement('a');
        	if (index == 0) {
        		a.href = 'data:application/octet-stream;base64,' + vm.service.firstcontract;
        		a.download = vm.service.servicename + "_Overarching_services_contract.pdf";
        	} else if (index == 1) {
        		a.href = 'data:application/octet-stream;base64,' + vm.service.secondcontract;
        		a.download = vm.service.servicename + "_SLA_contract.pdf";
        	} else if (index == 2) {
        		a.href = 'data:application/octet-stream;base64,' + vm.service.thirdcontract;
        		a.download = vm.service.servicename + "_Data_processing_contract.pdf";
        	}   		
    		a.target = '_blank';
    		a.style = "display: none";
    		a.click();
        }
        
        
        // Convierte un String en un array
        function convertStringToArray(string, separator) {
        	if (string != null) {
        		return string.split(separator);
        	}
        	return new Array();
        }
        
        
        // Convierte un array en un String
        function convertArrayToString(array, separator) {
        	if (array != null && array.length > 0) {
        		var string = "";
        		for (var i = 0;i < array.length;i++) {
        			string += array[i];
        			if (i < array.length - 1) {
        				string += separator;
        			}
        		}
        		return string;
        	}
        	return null;
        }
        
        
        // Inserta el servicio
        $scope.submitSaveService = function() {
        	// Transformar atributos con unidades y checklists
        	for (var i = 0;i < vm.service.attributes.length;i++) {
        		if (vm.service.attributes[i] != null) {
        			// Eliminar array auxiliar de unidades
        			delete vm.service.attributes[i].arrayunits;
        			
        			// Evaluar array auxiliar de checklists
        			if (vm.service.attributes[i].ischecklist) {
        				vm.service.attributes[i].checklistvalue =
        					convertArrayToString(vm.service.attributes[i].arraychecklist, ",");
            			
            			// Aplicar regla
            			var variable = vm.service.attributes[i].arraychecklist.length;
            			vm.service.attributes[i].value = eval(vm.service.attributes[i].checklistevalrule);
            			delete vm.service.attributes[i].arraychecklist;
            		}
        		}
        	}
        	
        	// Transformar atributo "Region"
        	if (vm.service.attributes[2].value != null && vm.service.attributes[2].value.length == 0) {
        		vm.service.attributes[2].value = null;
        	}
        	
        	// Guardar
            Service.save({attributes: vm.service.attributes}, vm.service, onSaveServiceSuccess, onSaveServiceError);
        }
        

        // Función de éxito en alta de servicio
        function onSaveServiceSuccess(result) {
        	$state.go("service");
        }


        // Función de error en alta de servicio
        function onSaveServiceError() {}
    }
})();
